package bin_trees;

import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Stack;

/**
 * A class to evaluate a given expression tree
 *
 */

public class EvaluateExpressionTree
{
    /**
     * Use a postorder traversal to store the content of an expression tree
     * inside a queue
     * @param et - the expression tree
     * @return the queue containing the content of the expression tree
     */
    public static Deque<VarBinNode> PostTraverse(VarBinNode et)
    {
        Deque<VarBinNode> q = new ArrayDeque<VarBinNode>(); 
        PostTraverse(et, q);
        return q;
    }


    /**
     * Helper recursive method to  store the content of an expression tree
     * inside a queue
     * @param et - the expression tree
     * @param q - the queue containing the content of the expression tree
     */
    public static void PostTraverse(VarBinNode et, Deque<VarBinNode> q)
    {
        if (et instanceof VarIntlNode)
        {
            PostTraverse(((VarIntlNode)et).leftchild(), q);
            PostTraverse(((VarIntlNode)et).rightchild(), q);
            q.offer(et);
        }
        else
        { // leaf node
            q.offer(et);
        }
    }


    /**
     * Evaluate (compute) the expression tree
     * @param q the queue containing the content of the expression tree
     * @return the value that is denoted by this expression tree
     */
    public static double Evaluate(Deque<VarBinNode> q)
    {
    	
    	Stack<VarLeafNode> s = new Stack<>();
    	while(!q.isEmpty()) {
    		VarBinNode n = q.poll();
    		if(n instanceof VarLeafNode) {
    			s.push((VarLeafNode)n);
    		}
    		else {
    			VarLeafNode op1 = s.pop();
    			VarLeafNode op2 = s.pop();
    			double value;
    			switch(((VarIntlNode)n).value()) {
    			case '+':
    				value = Double.parseDouble(op2.value()) +
    				Double.parseDouble(op1.value());
    				break;
    				
    			case '-':
    				value = Double.parseDouble(op2.value()) -
    				Double.parseDouble(op1.value());
    				break;
    				
    			case '*':
    				value = Double.parseDouble(op2.value()) *
    				Double.parseDouble(op1.value());
    				break;
    			case '/':
    				value = Double.parseDouble(op2.value()) /
    				Double.parseDouble(op1.value());
    				break;
				default:
					value= -9999;
    			}
    			// create a new leaf node
    			VarLeafNode newNode = new VarLeafNode(""+value);
    			s.push(newNode);
    		}
    		
    	}
    	return Double.parseDouble(s.pop().value());
    	
    }
}
